package com.cusc.nirvana.user.rnr.customer.service.impl;

import com.alibaba.nacos.client.naming.utils.CollectionUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.customer.dto.*;
import com.cusc.nirvana.user.rnr.customer.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.customer.service.IFileService;
import com.cusc.nirvana.user.rnr.customer.service.IVehicleCardService;
import com.cusc.nirvana.user.rnr.customer.util.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.fp.client.CardUnBindClient;
import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.client.VinCardClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author yubo
 * @since 2022-04-18 09:26
 */
@Service
@Slf4j
public class VehicleCardServiceImpl implements IVehicleCardService {

    @Autowired
    IFileService fileService;

    @Autowired
    VinCardClient vinCardClient;

    @Autowired
    CardUnBindClient cardUnBindClient;

    @Value("${fileTemplate.manufacturer.fileSize: 500}")
    private String fileSize;

    @Resource
    private FileSystemClient fileSystemClient;

    /**
     * 车卡关系和卡校验
     *
     * @param vinCardDTO
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCard(@Valid VinCardInfoDTO vinCardDTO) {
        //调用批量验证接口
        List<VinCardDTO> vinCardDTOS
                = vinCardDTO.getIccidList().stream().map(iccid -> new VinCardDTO(vinCardDTO.getVin(), iccid)).collect(Collectors.toList());

        return verifyVinCardBatch(vinCardDTOS);
    }

    /**
     * 批量车卡关系和卡校验
     *
     * @param verifyDTOs
     * @return
     */
    @Override
    public Response<VerifyVinCardResponseDTO> verifyVinCardBatch(List<VinCardDTO> verifyDTOs) {
        //进行vin和iccid的验证
        Response<VerifyVinCardResponseDTO> response = vinCardClient.verifyVinCard(verifyDTOs);
        if (!response.isSuccess()) {
            return response;
        }

        return Response.createSuccess(response.getData());
    }


    @Override
    public Response<UnBindIccidDTO> getUnBindIccidList(VinDTO vinDTO) {
        //根据vin查询所有iccid
        Response<VinCardInfoDTO> response = vinCardClient.queryVinCard(vinDTO.getVin());
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidList())) {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有已绑定的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vinDTO.getVin()));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        UnBindIccidDTO unBindIccidDTO = new UnBindIccidDTO();

        String rnrId;
        //过滤掉已经实名的iccid
        List<String> iccidList = response.getData().getIccidList();
        if (!CollectionUtils.isEmpty(cardListResponse.getData())) {
            Set<String> bindIccids = cardListResponse.getData().stream()
                    .filter(card -> card.getRnrStatus() == RnrStatus.RNR.getCode())
                    .map(card -> card.getIccid()).collect(Collectors.toSet());
            iccidList = iccidList.stream().filter(id -> !bindIccids.contains(id)).distinct().collect(Collectors.toList());
            rnrId = cardListResponse.getData().get(0).getRnrId();
        } else {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有已实名的卡", ResponseCode.INVALID_DATA.getCode());
        }
        if (CollectionUtils.isEmpty(iccidList)) {
            return Response.createError("VIN【" + vinDTO.getVin() + "】没有未实名的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //获取实名信息
        Response<MgRnrInfoDTO> rnrInfo = cardUnBindClient.getRnrInfo(new FpVehicleCardRnrDTO().setRnrId(rnrId));
        if (!rnrInfo.isSuccess()) {
            return Response.createError(rnrInfo.getMsg(), rnrInfo.getCode());
        }
        MgRnrInfoDTO rnrInfoDTO = rnrInfo.getData();
        if (rnrInfoDTO == null) {
            return Response.createError("未获取到对应的实名信息", ResponseCode.INVALID_DATA.getCode());
        }

        unBindIccidDTO.setIccidList(iccidList);
        unBindIccidDTO.setVin(vinDTO.getVin());
        unBindIccidDTO.setGender(rnrInfoDTO.getGender());
        unBindIccidDTO.setRnrId(rnrInfoDTO.getUuid());
        unBindIccidDTO.setName(DesensitizationUtil.desensitizeName(rnrInfoDTO.getFullName()));
        unBindIccidDTO.setPhone(DesensitizationUtil.desensitizePhone(rnrInfoDTO.getPhone()));

        return Response.createSuccess(unBindIccidDTO);
    }

    @Override
    public Response<VehicleCardRnrDTO> getBindIccidList(VinDTO vinDTO) {
        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vinDTO.getVin()));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        VehicleCardRnrDTO vehicleCardRnrDTO = new VehicleCardRnrDTO();
        if(!CollectionUtils.isEmpty(cardListResponse.getData())){
            List<String> iccids = cardListResponse.getData().stream()
                    .filter(card -> card.getRnrStatus() == RnrStatus.RNR.getCode())
                    .map(c -> c.getIccid()).collect(Collectors.toList());
            vehicleCardRnrDTO.setVin(vinDTO.getVin());
            vehicleCardRnrDTO.setIccidList(iccids);
            vehicleCardRnrDTO.setRnrId(cardListResponse.getData().get(0).getRnrId());
        }

        return Response.createSuccess(vehicleCardRnrDTO);
    }

    /**
     * 新车实名查询iccid，如果已经有实名信息，提醒走一车多卡
     * @param vin
     * @return
     */
    @Override
    public Response<IccidListDTO> getNewCarUnBindIccidList(String vin) {
        //根据vin查询所有iccid
        Response<VinCardInfoDTO> response = vinCardClient.queryVinCard(vin);
        if (!response.isSuccess()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        if (response.getData() == null || CollectionUtils.isEmpty(response.getData().getIccidList())) {
            return Response.createError("VIN【" + vin + "】没有已绑定的卡", ResponseCode.INVALID_DATA.getCode());
        }

        //根据vin查询所有已经实名的iccid
        Response<List<MgRnrCardInfoDTO>> cardListResponse = cardUnBindClient.getCardList(new FpVehicleCardRnrDTO().setVin(vin));
        if (!cardListResponse.isSuccess()) {
            return Response.createError(cardListResponse.getMsg(), cardListResponse.getCode());
        }

        if(!CollectionUtils.isEmpty(cardListResponse.getData())){
            return Response.createError("VIN【" + vin + "】存在已实名的卡，请通过一车多卡进行绑定",ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess(response.getData());

    }


}
